/*---------------------------------------------------------------------------*/
//       Author : hiyohiyo
//         Mail : hiyohiyo@crystalmark.info
//          Web : http://crystalmark.info/
//      License : The modified BSD license
//
//                                Copyright 2008 hiyohiyo. All rights reserved.
/*---------------------------------------------------------------------------*/
// Reference : http://www.usefullcode.net/2007/02/hddsmart.html (ja)

#pragma once

#include "winioctl.h"
#include "atlstr.h"

class CAtaSmart
{
public:
	static const int MAX_DISK = 16;// FIX
	static const int MAX_ATTRIBUTE = 30; // FIX
	static const int MAX_SEARCH_PHYSICAL_DRIVE = 32;
	static const int MAX_SEARCH_SCSI_PORT = 16;
	static const int MAX_SEARCH_SCSI_TARGET_ID = 8;

public:
	CAtaSmart();
	virtual ~CAtaSmart();

	enum SMART_STATUS
	{
		SMART_STATUS_NO_CHANGE = 0,
		SMART_STATUS_MINOR_CHANGE,
		SMART_STATUS_MAJOR_CHANGE
	};

	enum TRANSFER_MODE
	{
		UNKNOWN = 0,
		PIO,
		PIO_DMA,
		ULTRA_DMA_16,
		ULTRA_DMA_25,
		ULTRA_DMA_33,
		ULTRA_DMA_44,
		ULTRA_DMA_66,
		ULTRA_DMA_100,
		ULTRA_DMA_133,
		SATA_150,
		SATA_300,
		SATA_600
	};

	enum DISK_STATUS
	{
		DISK_STATUS_UNKNOWN = 0,
		DISK_STATUS_GOOD,
		DISK_STATUS_CAUTION,
		DISK_STATUS_BAD
	};

	enum POWER_ON_HOURS_UNIT
	{
		POWER_ON_UNKNOWN = 0,
		POWER_ON_HOURS,
		POWER_ON_MINUTES,
		POWER_ON_HALF_MINUTES,
		POWER_ON_SECONDS,
	};

	enum ACCESS_TYPE
	{
		PHYSICALDRIVE = 0x0,
		SCSI_MINIPORT
	};

protected:
	enum IO_CONTROL_CODE
	{
		DFP_SEND_DRIVE_COMMAND	= 0x0007C084,
		DFP_RECEIVE_DRIVE_DATA	= 0x0007C088,
		IOCTL_SCSI_MINIPORT     = 0x0004D008,
	};

#define	FILE_DEVICE_SCSI							0x0000001b
#define	IOCTL_SCSI_MINIPORT_IDENTIFY				((FILE_DEVICE_SCSI << 16) + 0x0501)
#define	IOCTL_SCSI_MINIPORT_READ_SMART_ATTRIBS		((FILE_DEVICE_SCSI << 16) + 0x0502)
#define IOCTL_SCSI_MINIPORT_READ_SMART_THRESHOLDS	((FILE_DEVICE_SCSI << 16) + 0x0503)
#define IOCTL_SCSI_MINIPORT_ENABLE_SMART			((FILE_DEVICE_SCSI << 16) + 0x0504)
#define IOCTL_SCSI_MINIPORT_DISABLE_SMART			((FILE_DEVICE_SCSI << 16) + 0x0505)

#pragma pack(push,1)

	struct	IDENTIFY_DEVICE_OUTDATA
	{
		SENDCMDOUTPARAMS	SendCmdOutParam;
		BYTE				Data[IDENTIFY_BUFFER_SIZE - 1];
	};

	struct	SMART_READ_DATA_OUTDATA
	{
		SENDCMDOUTPARAMS	SendCmdOutParam;
		BYTE				Data[READ_ATTRIBUTE_BUFFER_SIZE - 1];
	};

	typedef	struct SMART_ATTRIBUTE
	{
		BYTE	Id;
		WORD	StatusFlags;
		BYTE	CurrentValue;
		BYTE	WorstValue;
		BYTE	RawValue[6];
		BYTE	Reserved;
	};

	typedef	struct SMART_THRESHOLD
	{
		BYTE	Id;
		BYTE	ThresholdValue;
		BYTE	Reserved[10];
	};


	typedef struct SRB_IO_CONTROL
	{
	   ULONG	HeaderLength;
	   UCHAR	Signature[8];
	   ULONG	Timeout;
	   ULONG	ControlCode;
	   ULONG	ReturnCode;
	   ULONG	Length;
	};

	typedef struct SRB_IO_COMMAND{
		SRB_IO_CONTROL	Cntrol;
		IDEREGS			IdeRegs;
		BYTE			Data[512];
	};

	struct IDENTIFY_DEVICE
	{
		WORD		GeneralConfiguration;				//0
		WORD		LogicalCylinders;					//1	Obsolete
		WORD		SpecificConfiguration;				//2
		WORD		LlogicalHeads;						//3 Obsolete
		WORD		Retired1[2];						//4-5
		WORD		LogicalSectors;						//6 Obsolete
		DWORD		ReservedForCompactFlash;			//7-8
		WORD		Retired2;							//9
		CHAR		SerialNumber[20];					//10-19
		WORD		Retired3;							//20
		WORD		BufferSize;							//21 Obsolete
		WORD		Obsolute4;							//22
		CHAR		FirmwareRev[8];						//23-26
		CHAR		Model[40];							//27-46
		WORD		MaxNumPerInterupt;					//47
		WORD		Reserved1;							//48
		WORD		Capabilities1;						//49
		WORD		Capabilities2;						//50
		DWORD		Obsolute5;							//51-52
		WORD		Field88and7064;						//53
		WORD		Obsolute6[5];						//54-58
		WORD		MultSectorStuff;					//59
		DWORD		TotalAddressableSectors;			//60-61
		WORD		Obsolute7;							//62
		WORD		MultiWordDma;						//63
		WORD		PioMode;							//64
		WORD		MinMultiwordDmaCycleTime;			//65
		WORD		RecommendedMultiwordDmaCycleTime;	//66
		WORD		MinPioCycleTimewoFlowCtrl;			//67
		WORD		MinPioCycleTimeWithFlowCtrl;		//68
		WORD		Reserved2[6];						//69-74
		WORD		QueueDepth;							//75
		WORD		SerialAtaCapabilities;				//76
		WORD		ReservedForFutureSerialAta;			//77
		WORD		SerialAtaFeaturesSupported;			//78
		WORD		SerialAtaFeaturesEnabled;			//79
		WORD		MajorVersion;						//80
		WORD		MinorVersion;						//81
		WORD		CommandSetSupported1;				//82
		WORD		CommandSetSupported2;				//83
		WORD		CommandSetSupported3;				//84
		WORD		CommandSetEnabled1;					//85
		WORD		CommandSetEnabled2;					//86
		WORD		CommandSetDefault;					//87
		WORD		UltraDmaMode;						//88
		WORD		TimeReqForSecurityErase;			//89
		WORD		TimeReqForEnhancedSecure;			//90
		WORD		CurrentPowerManagement;				//91
		WORD		MasterPasswordRevision;				//92
		WORD		HardwareResetResult;				//93
		WORD		Acoustricmanagement;				//94
		WORD		StreamMinRequestSize;				//95
		WORD		StreamingTimeDma;					//96
		WORD		StreamingAccessLatency;				//97
		DWORD		StreamingPerformance;				//98-99
		ULONGLONG	MaxUserLba;							//100-103
		WORD		StremingTimePio;					//104
		WORD		Reserved3;							//105
		WORD		SectorSize;							//106
		WORD		InterSeekDelay;						//107
		WORD		IeeeOui;							//108
		WORD		UniqueId3;							//109
		WORD		UniqueId2;							//110
		WORD		UniqueId1;							//111
		WORD		Reserved4[4];						//112-115
		WORD		Reserved5;							//116
		DWORD		WordsPerLogicalSector;				//117-118
		WORD		Reserved6[8];						//119-126
		WORD		RemovableMediaStatus;				//127
		WORD		SecurityStatus;						//128
		WORD		VendorSpecific[31];					//129-159
		WORD		CfaPowerMode1;						//160
		WORD		Reserved7[15];						//161-175
		CHAR		CurrentMediaSerialNo[60];			//176-205
		WORD		SctCommandTransport;				//206  254
		WORD		ReservedForCeAta1[2];				//207-208
		WORD		AlignmentOfLogicalBlocks;			//209
		DWORD		WriteReadVerifySectorCountMode3;	//210-211
		DWORD		WriteReadVerifySectorCountMode2;	//212-213
		WORD		NvCacheCapabilities;				//214
		DWORD		NvCacheSizeLogicalBlocks;			//215-216
		WORD		NominalMediaRotationRate;			//217
		WORD		Reserved8;							//218
		WORD		NvCacheOptions1;					//219
		WORD		NvCacheOptions2;					//220
		WORD		Reserved9;							//221
		WORD		TransportMajorVersionNumber;		//222
		WORD		TransportMinorVersionNumber;		//223
		WORD		ReservedForCeAta2[10];				//224-233
		WORD		MinimumBlocksPerDownloadMicrocode;	//234
		WORD		MaximumBlocksPerDownloadMicrocode;	//235
		WORD		Reserved10[19];						//236-254
		WORD		IntegrityWord;						//255
	};
#pragma	pack(pop)

public:
	DWORD UpdateSmartInfo(DWORD index);
	BOOL Init(BOOL useWmi, BOOL advancedDiskSearch);
	BOOL MeasuredTimeUnit();
	DWORD GetPowerOnHours(DWORD rawValue, DWORD timeUnitType);
	DWORD GetPowerOnHoursEx(DWORD index, DWORD timeUnitType);
	
	struct ATA_SMART_INFO
	{
		IDENTIFY_DEVICE		IdentifyDevice;
		SMART_ATTRIBUTE		Attribute[MAX_ATTRIBUTE];
		SMART_THRESHOLD		Threshold[MAX_ATTRIBUTE];

		BOOL				IsSmartEnabled;
		BOOL				IsCheckSumError;
		BOOL				IsWord88;
		BOOL				IsWord64_76;

		BOOL				IsSmartSupported;
		BOOL				IsLba48Supported;
		BOOL				IsAamSupported;
		BOOL				IsNcqSupported;
		BOOL				IsNvCacheSupported;
		BOOL				IsMaxtorMinute;

		INT					PhysicalDriveId;
		INT					ScsiPort;
		INT					ScsiTargetId;
		INT					AccessType;

		DWORD				TotalDiskSize;
		DWORD				Cylinder;
		DWORD				Head;
		DWORD				Sector;
		DWORD				DiskSizeChs;
		DWORD				DiskSizeLba28;
		DWORD				DiskSizeLba48;
		DWORD				BufferSize;
		ULONGLONG			NvCacheSize;
		DWORD				TransferModeType;
		DWORD				DetectedTimeUnitType;
		DWORD				MeasuredTimeUnitType;
		DWORD				AttributeCount;
		DWORD				DetectedPowerOnHours;
		DWORD				MeasuredPowerOnHours;
		DWORD				PowerOnRawValue;
		DWORD				PowerOnStartRawValue;
		DWORD				PowerOnCount;
		DWORD				Temperature;
		double				Speed;

		DWORD				Major;
		DWORD				Minor;

		DWORD				DiskStatus;
		DWORD				DriveLetterMap;
		// 
		DWORD				AlarmTemperature;
		BOOL				AlarmHealthStatus;

		CAtlString			SerialNumber;
		CAtlString			SerialNumberReverse;
		CAtlString			FirmwareRev;
		CAtlString			FirmwareRevReverse;
		CAtlString			Model;
		CAtlString			ModelReverse;
		CAtlString			ModelWmi;
		CAtlString			DriveMap;
		CAtlString			MaxTransferMode;
		CAtlString			CurrentTransferMode;
		CAtlString			MajorVersion;
		CAtlString			MinorVersion;
		CAtlString			Interface;
	};

	CArray<ATA_SMART_INFO, ATA_SMART_INFO> vars;

	CStringArray m_IdeController;
	CStringArray m_ScsiController;
	CString m_ControllerMap;

	BOOL IsEnabledWmi;
	DWORD MeasuredGetTickCount;

protected:
	CString m_SerialNumberA_Z[26];

	BOOL GetDiskInfo(INT physicalDriveId, INT scsiPort, INT scsiTargetId);
	BOOL AddDisk(INT PhysicalDriveId, INT ScsiPort, INT scsiTargetId, INT AccessType, IDENTIFY_DEVICE* identify);
	DWORD CheckSmartAttributeUpdate(DWORD index, SMART_ATTRIBUTE* pre, SMART_ATTRIBUTE* cur);

	VOID InitAtaInfo();
	VOID InitAtaInfoByWmi();
	VOID InitStruct();
	VOID ChangeByteOrder(PCHAR str, DWORD length);
	BOOL CheckAsciiStringError(PCHAR str, DWORD length);
	HANDLE GetIoCtrlHandle(BYTE index);

	BOOL DoIdentifyDevicePd(INT physicalDriveId, IDENTIFY_DEVICE* identify);
	BOOL GetSmartAttributePd(INT physicalDriveId, ATA_SMART_INFO* asi);
	BOOL GetSmartThresholdPd(INT physicalDriveId, ATA_SMART_INFO* asi);
	BOOL ControlSmartStatusPd(INT physicalDriveId, BYTE command);

	BOOL DoIdentifyDeviceScsi(INT scsiPort, INT scsiTargetId, IDENTIFY_DEVICE* identify);
	BOOL GetSmartAttributeScsi(INT scsiPort, INT scsiTargetId, ATA_SMART_INFO* asi);
	BOOL GetSmartThresholdScsi(INT scsiPort, INT scsiTargetId, ATA_SMART_INFO* asi);
	BOOL ControlSmartStatusScsi(INT scsiPort, INT scsiTargetId, BYTE command);

	DWORD CheckDiskStatus(SMART_ATTRIBUTE* attribute, SMART_THRESHOLD* threshold, DWORD attributeCount);

	DWORD GetTransferMode(WORD w63, WORD w76, WORD w88, CAtlString &currentTransferMode, CAtlString &maxTransferMode, CAtlString &interfaceType);
	DWORD GetTimeUnitType(CAtlString model, CAtlString firmware, DWORD major, DWORD transferMode);
	DWORD GetAtaMajorVersion(WORD w80, CAtlString &majorVersion);
//	DWORD GetMaxtorPowerOnHours(DWORD currentValue, DWORD rawValue);

	static int Compare(const void *p1, const void *p2);
	void DebugPrint(CAtlString cstr);
};
